﻿Imports System.ComponentModel

''' <summary>
''' 独自の背景、移動・サイズ変更処理をサポートするためのコントロール
''' </summary>
''' <remarks></remarks>
Public Class PopupForm
#Region "定義・メンバー変数"
    Private Enum BorderPoint
        None
        NearTopRight
        NearTopLeft
        NearTop
        NearBottomRight
        NearBottomLeft
        NearBottom
        NearRight
        NearLeft
    End Enum

    Private prop_bSizAble As Boolean = True
    Private prop_bMovAble As Boolean = True
    Private prop_siMinWindow As Size = New Size(60, 20)

    Private prop_1_1Image As Image
    Private prop_1_2Image As Image
    Private prop_1_3Image As Image
    Private prop_2_1Image As Image
    Private prop_2_2Image As Image
    Private prop_2_3Image As Image
    Private prop_3_1Image As Image
    Private prop_3_2Image As Image
    Private prop_3_3Image As Image

    Private Const SIZEABLEMERGEN As Integer = 8

    Private m_bpSizing As BorderPoint = BorderPoint.None
    Private m_bMoving As Boolean = False
    Private m_poiMouse As Point
#End Region

#Region "イベント関数"
    Private Sub PopupForm_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles MyBase.Paint
        If prop_1_1Image Is Nothing Or prop_1_2Image Is Nothing Or prop_1_3Image Is Nothing _
           Or prop_2_1Image Is Nothing Or prop_2_2Image Is Nothing Or prop_2_3Image Is Nothing _
           Or prop_3_1Image Is Nothing Or prop_3_2Image Is Nothing Or prop_3_3Image Is Nothing Then
            Dim br As Brush = New SolidBrush(Me.BackColor)
            e.Graphics.FillRectangle(br, 0, 0, Me.Width, Me.Height)
        Else
            e.Graphics.DrawImage(prop_1_1Image, _
              0, 0, prop_1_1Image.Width, prop_1_1Image.Height)
            e.Graphics.DrawImage(prop_1_2Image, 0, prop_1_1Image.Height, prop_1_2Image.Width, Me.Height - prop_1_1Image.Height - prop_1_3Image.Height + 1)
            e.Graphics.DrawImage(prop_1_3Image, _
              0, Me.Height - prop_1_3Image.Height, prop_1_3Image.Width, prop_1_3Image.Height)

            e.Graphics.DrawImage(prop_2_1Image, _
              prop_1_1Image.Width, 0, _
              Me.Width - prop_1_1Image.Width - prop_3_1Image.Width + 1, prop_1_1Image.Height)
            e.Graphics.DrawImage(prop_2_2Image, _
              prop_1_2Image.Width, prop_2_1Image.Height, _
              Me.Width - prop_1_2Image.Width - prop_3_2Image.Width + 2, Me.Height - prop_2_1Image.Height - prop_2_3Image.Height + 2)

            e.Graphics.DrawImage(prop_2_3Image, _
              prop_1_3Image.Width, Me.Height - prop_2_3Image.Height, _
              Me.Width - prop_1_3Image.Width - prop_3_3Image.Width + 1, prop_2_3Image.Height)

            e.Graphics.DrawImage(prop_3_1Image, _
              Me.Width - prop_3_1Image.Width, 0, _
              prop_3_1Image.Width, prop_3_1Image.Height)
            e.Graphics.DrawImage(prop_3_2Image, _
              Me.Width - prop_3_2Image.Width, prop_3_1Image.Height, _
              prop_3_2Image.Width, Me.Height - prop_3_1Image.Height - prop_3_3Image.Height + 1)
            e.Graphics.DrawImage(prop_3_3Image, _
              Me.Width - prop_3_3Image.Width, Me.Height - prop_3_3Image.Height, _
              prop_3_3Image.Width, prop_3_3Image.Height)

        End If

    End Sub

    ''' <summary>
    ''' マウスの移動処理。サイズ変更や移動を制御する
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PopupForm_MouseMove(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseMove
        Dim bp As BorderPoint
        Dim si As Size
        Dim siTemp As Size

        If Not m_bpSizing = BorderPoint.None Then ' サイズ変更中のマウス移動
            Dim bChanged As Boolean = False

            If Math.Abs(e.Location.X - m_poiMouse.X) < 8 Then
                If Math.Abs(e.Location.Y - m_poiMouse.Y) < 8 Then
                    Return
                End If
            End If

            Me.SuspendLayout()

            Select Case m_bpSizing
                Case BorderPoint.NearTopRight
                    si = New Size(e.Location.X - m_poiMouse.X, -e.Location.Y)
                    siTemp = Me.Size + si
                    If siTemp.Width < prop_siMinWindow.Width Then
                        si.Width = prop_siMinWindow.Width - Me.Width
                        siTemp.Width = prop_siMinWindow.Width
                    End If
                    If siTemp.Height < prop_siMinWindow.Height Then
                        si.Height = prop_siMinWindow.Height - Me.Height
                        siTemp.Height = prop_siMinWindow.Height
                    End If
                    If siTemp.Width = Me.Width And siTemp.Height = Me.Height Then
                        Exit Select
                    End If
                    If si.IsEmpty Then
                        Exit Select
                    End If

                    bChanged = True
                    Me.Parent.Location += New Point(0, -si.Height)
                    Me.Size += si
                    Me.Parent.Size += si
                    m_poiMouse = e.Location

                Case BorderPoint.NearTopLeft
                    si = New Size(-e.Location.X, -e.Location.Y)
                    siTemp = Me.Size + si
                    If siTemp.Width < prop_siMinWindow.Width Then
                        si.Width = prop_siMinWindow.Width - Me.Width
                        siTemp.Width = prop_siMinWindow.Width
                    End If
                    If siTemp.Height < prop_siMinWindow.Height Then
                        si.Height = prop_siMinWindow.Height - Me.Height
                        siTemp.Height = prop_siMinWindow.Height
                    End If
                    If siTemp.Width = Me.Width And siTemp.Height = Me.Height Then
                        Exit Select
                    End If
                    If si.IsEmpty Then
                        Exit Select
                    End If

                    bChanged = True
                    Me.Parent.Location += New Point(-si.Width, -si.Height)
                    Me.Size += si
                    Me.Parent.Size += si
                    m_poiMouse = e.Location

                Case BorderPoint.NearTop
                    si = New Size(0, -e.Location.Y)
                    siTemp = Me.Size + si
                    If siTemp.Width < prop_siMinWindow.Width Then
                        si.Width = prop_siMinWindow.Width - Me.Width
                        siTemp.Width = prop_siMinWindow.Width
                    End If
                    If siTemp.Height < prop_siMinWindow.Height Then
                        si.Height = prop_siMinWindow.Height - Me.Height
                        siTemp.Height = prop_siMinWindow.Height
                    End If
                    If siTemp.Width = Me.Width And siTemp.Height = Me.Height Then
                        Exit Select
                    End If
                    If si.IsEmpty Then
                        Exit Select
                    End If

                    bChanged = True
                    Me.Parent.Location += New Point(si.Width, -si.Height)
                    Me.Size += si
                    Me.Parent.Size += si
                    m_poiMouse = e.Location

                Case BorderPoint.NearBottomRight
                    si = New Size(e.Location.X - m_poiMouse.X, e.Location.Y - m_poiMouse.Y)
                    siTemp = Me.Size + si
                    If siTemp.Width < prop_siMinWindow.Width Then
                        si.Width = prop_siMinWindow.Width - Me.Width
                        siTemp.Width = prop_siMinWindow.Width
                    End If
                    If siTemp.Height < prop_siMinWindow.Height Then
                        si.Height = prop_siMinWindow.Height - Me.Height
                        siTemp.Height = prop_siMinWindow.Height
                    End If
                    If siTemp.Width = Me.Width And siTemp.Height = Me.Height Then
                        Exit Select
                    End If
                    If si.IsEmpty Then
                        Exit Select
                    End If

                    bChanged = True
                    Me.Size += si
                    Me.Parent.Size += si
                    m_poiMouse = e.Location

                Case BorderPoint.NearBottomLeft
                    si = New Size(-e.Location.X, e.Location.Y - m_poiMouse.Y)
                    siTemp = Me.Size + si
                    If siTemp.Width < prop_siMinWindow.Width Then
                        si.Width = prop_siMinWindow.Width - Me.Width
                        siTemp.Width = prop_siMinWindow.Width
                    End If
                    If siTemp.Height < prop_siMinWindow.Height Then
                        si.Height = prop_siMinWindow.Height - Me.Height
                        siTemp.Height = prop_siMinWindow.Height
                    End If
                    If siTemp.Width = Me.Width And siTemp.Height = Me.Height Then
                        Exit Select
                    End If
                    If si.IsEmpty Then
                        Exit Select
                    End If

                    bChanged = True
                    Me.Parent.Location += New Point(-si.Width, 0)
                    Me.Size += si
                    Me.Parent.Size += si
                    m_poiMouse = e.Location

                Case BorderPoint.NearBottom
                    si = New Size(0, e.Location.Y - m_poiMouse.Y)
                    siTemp = Me.Size + si
                    If siTemp.Width < prop_siMinWindow.Width Then
                        si.Width = prop_siMinWindow.Width - Me.Width
                        siTemp.Width = prop_siMinWindow.Width
                    End If
                    If siTemp.Height < prop_siMinWindow.Height Then
                        si.Height = prop_siMinWindow.Height - Me.Height
                        siTemp.Height = prop_siMinWindow.Height
                    End If
                    If siTemp.Width = Me.Width And siTemp.Height = Me.Height Then
                        Exit Select
                    End If
                    If si.IsEmpty Then
                        Exit Select
                    End If

                    bChanged = True
                    Me.Size += si
                    Me.Parent.Size += si
                    m_poiMouse = e.Location

                Case BorderPoint.NearRight
                    si = New Size(e.Location.X - m_poiMouse.X, 0)
                    siTemp = Me.Size + si
                    If siTemp.Width < prop_siMinWindow.Width Then
                        si.Width = prop_siMinWindow.Width - Me.Width
                        siTemp.Width = prop_siMinWindow.Width
                    End If
                    If siTemp.Height < prop_siMinWindow.Height Then
                        si.Height = prop_siMinWindow.Height - Me.Height
                        siTemp.Height = prop_siMinWindow.Height
                    End If
                    If siTemp.Width = Me.Width And siTemp.Height = Me.Height Then
                        Exit Select
                    End If
                    If si.IsEmpty Then
                        Exit Select
                    End If

                    bChanged = True
                    Me.Size += si
                    Me.Parent.Size += si
                    m_poiMouse = e.Location

                Case BorderPoint.NearLeft
                    si = New Size(-e.Location.X, 0)
                    siTemp = Me.Size + si
                    If siTemp.Width < prop_siMinWindow.Width Then
                        si.Width = prop_siMinWindow.Width - Me.Width
                        siTemp.Width = prop_siMinWindow.Width
                    End If
                    If siTemp.Height < prop_siMinWindow.Height Then
                        si.Height = prop_siMinWindow.Height - Me.Height
                        siTemp.Height = prop_siMinWindow.Height
                    End If
                    If siTemp.Width = Me.Width And siTemp.Height = Me.Height Then
                        Exit Select
                    End If
                    If si.IsEmpty Then
                        Exit Select
                    End If

                    bChanged = True
                    Me.Parent.Location += New Point(-si.Width, -si.Height)
                    Me.Size += si
                    Me.Parent.Size += si
                    m_poiMouse = e.Location

                Case Else
            End Select

            Me.ResumeLayout()
            If bChanged = True Then
                Me.Invalidate()
            End If


        ElseIf m_bMoving Then ' ウィンドウ移動中のマウス移動
            ' ウィンドウ移動は親のフォームで行う
            Me.Parent.Location += (e.Location - m_poiMouse)
        Else
            If prop_bSizAble Then ' 移動・サイズ変更中でない時、サイズ変更可能ならばカーソル制御をする
                bp = CheckBorderPoint(e.Location, SIZEABLEMERGEN)

                Select Case bp
                    Case BorderPoint.NearBottom, BorderPoint.NearTop
                        Me.Cursor = Cursors.SizeNS

                    Case BorderPoint.NearLeft, BorderPoint.NearRight
                        Me.Cursor = Cursors.SizeWE

                    Case BorderPoint.NearTopRight, BorderPoint.NearBottomLeft
                        Me.Cursor = Cursors.SizeNESW

                    Case BorderPoint.NearTopLeft, BorderPoint.NearBottomRight
                        Me.Cursor = Cursors.SizeNWSE

                    Case Else
                        Me.Cursor = Cursors.Arrow
                End Select
            End If

        End If

    End Sub

    ''' <summary>
    ''' マウス押下処理
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PopupForm_MouseDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
        Dim bp As BorderPoint

        If m_bMoving Then
            Return
        End If
        If Not m_bpSizing = BorderPoint.None Then
            Return
        End If
        bp = CheckBorderPoint(e.Location, SIZEABLEMERGEN)
        m_poiMouse = e.Location
        If bp = BorderPoint.None Then
            If prop_bMovAble Then
                m_bMoving = True
            End If
        Else
            If prop_bSizAble Then
                m_bpSizing = bp
            End If
        End If
    End Sub

    ''' <summary>
    ''' マウス押下解除処理
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PopupForm_MouseUp(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseUp
        m_poiMouse = e.Location
        m_bMoving = False
        m_bpSizing = BorderPoint.None
    End Sub

    ''' <summary>
    ''' マウスカーソルがコントロールから離れたらカーソルを標準に戻す
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PopupForm_MouseLeave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.MouseLeave
        Me.Cursor = Cursors.Arrow
    End Sub
#End Region

#Region "ユーザー関数"
    ''' <summary>
    ''' ポイントがウィンドウ境界の側にあるか。
    ''' </summary>
    ''' <param name="pos"></param>
    ''' <param name="round"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function CheckBorderPoint(ByVal pos As Point, ByVal round As Integer) As BorderPoint
        Dim ret As BorderPoint
        ret = BorderPoint.None
        If (pos.X >= Me.Size.Width - round) And (pos.X <= Me.Size.Width) Then
            If (pos.Y >= Me.Size.Height - round) And (pos.Y <= Me.Size.Height) Then
                ret = BorderPoint.NearBottomRight
            ElseIf (pos.Y >= 0) And (pos.Y <= round) Then
                ret = BorderPoint.NearTopRight
            Else
                ret = BorderPoint.NearRight
            End If
        ElseIf (pos.X >= 0) And (pos.X <= round) Then
            If (pos.Y >= Me.Size.Height - round) And (pos.Y <= Me.Size.Height) Then
                ret = BorderPoint.NearBottomLeft
            ElseIf (pos.Y >= 0) And (pos.Y <= round) Then
                ret = BorderPoint.NearTopLeft
            Else
                ret = BorderPoint.NearLeft
            End If
        Else
            If (pos.Y >= Me.Size.Height - round) And (pos.Y <= Me.Size.Height) Then
                ret = BorderPoint.NearBottom
            ElseIf (pos.Y >= 0) And (pos.Y <= round) Then
                ret = BorderPoint.NearTop
            Else
                ret = BorderPoint.None
            End If
        End If

        Return ret
    End Function
#End Region

#Region "プロパティ"
    ''' <summary>
    ''' サイズ変更を可能とするかの設定・取得
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PopupForm")> _
    <Description("サイズ変更を可能とするかの設定・取得")> _
    Public Property SizAble() As Boolean
        Get
            Return prop_bSizAble
        End Get
        Set(ByVal value As Boolean)
            prop_bSizAble = value
        End Set
    End Property

    ''' <summary>
    ''' フォームをドラッグしての移動を可能とするかの設定・取得
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PopupForm")> _
    <Description("フォームをドラッグしての移動を可能とするかの設定・取得")> _
    Public Property MovAble() As Boolean
        Get
            Return prop_bMovAble
        End Get
        Set(ByVal value As Boolean)
            prop_bMovAble = value
        End Set
    End Property

    ''' <summary>
    ''' 最小ウィンドウサイズの設定・取得
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PopupForm")> _
    <Description("最小ウィンドウサイズの設定・取得")> _
    Public Property MinWindowSize() As Size
        Get
            Return prop_siMinWindow
        End Get
        Set(ByVal value As Size)
            prop_siMinWindow = value
        End Set
    End Property

    <Category("PopupForm")> _
    <Description("左上隅のイメージ")> _
    <DefaultValue(GetType(Image), Nothing)> _
    Public Property Image1_1() As Image
        Get
            Return prop_1_1Image
        End Get
        Set(ByVal value As Image)
            prop_1_1Image = value
        End Set
    End Property

    <Category("PopupForm")> _
    <Description("左中央のイメージ")> _
    <DefaultValue(GetType(Image), Nothing)> _
    Public Property Image1_2() As Image
        Get
            Return prop_1_2Image
        End Get
        Set(ByVal value As Image)
            prop_1_2Image = value
        End Set
    End Property

    <Category("PopupForm")> _
    <Description("左下隅のイメージ")> _
    <DefaultValue(GetType(Image), Nothing)> _
    Public Property Image1_3() As Image
        Get
            Return prop_1_3Image
        End Get
        Set(ByVal value As Image)
            prop_1_3Image = value
        End Set
    End Property

    <Category("PopupForm")> _
    <Description("上隅のイメージ")> _
    <DefaultValue(GetType(Image), Nothing)> _
    Public Property Image2_1() As Image
        Get
            Return prop_2_1Image
        End Get
        Set(ByVal value As Image)
            prop_2_1Image = value
        End Set
    End Property

    <Category("PopupForm")> _
    <Description("中央のイメージ")> _
    <DefaultValue(GetType(Image), Nothing)> _
    Public Property Image2_2() As Image
        Get
            Return prop_2_2Image
        End Get
        Set(ByVal value As Image)
            prop_2_2Image = value
        End Set
    End Property

    <Category("PopupForm")> _
    <Description("下隅のイメージ")> _
    <DefaultValue(GetType(Image), Nothing)> _
    Public Property Image2_3() As Image
        Get
            Return prop_2_3Image
        End Get
        Set(ByVal value As Image)
            prop_2_3Image = value
        End Set
    End Property

    <Category("PopupForm")> _
    <Description("右上隅のイメージ")> _
    <DefaultValue(GetType(Image), Nothing)> _
    Public Property Image3_1() As Image
        Get
            Return prop_3_1Image
        End Get
        Set(ByVal value As Image)
            prop_3_1Image = value
        End Set
    End Property

    <Category("PopupForm")> _
    <Description("右中央のイメージ")> _
    <DefaultValue(GetType(Image), Nothing)> _
    Public Property Image3_2() As Image
        Get
            Return prop_3_2Image
        End Get
        Set(ByVal value As Image)
            prop_3_2Image = value
        End Set
    End Property

    <Category("PopupForm")> _
    <Description("右下隅のイメージ")> _
    <DefaultValue(GetType(Image), Nothing)> _
    Public Property Image3_3() As Image
        Get
            Return prop_3_3Image
        End Get
        Set(ByVal value As Image)
            prop_3_3Image = value
        End Set
    End Property

#End Region

End Class
