﻿Imports System.Windows.Forms

Public Class PhotoViewerForm
    Public Event ImageRefreshEvent()


    Friend Sub New()

        ' この呼び出しは、Windows フォーム デザイナで必要です。
        InitializeComponent()

        Me.SetStyle(ControlStyles.Opaque, True)
        ' InitializeComponent() 呼び出しの後で初期化を追加します。
    End Sub

    ''' <summary>
    ''' コンストラクタ。ファイルのリストと最初に表示する位置を受け取る
    ''' </summary>
    ''' <param name="FileList"></param>
    ''' <param name="Item"></param>
    ''' <remarks></remarks>
    Friend Sub New(ByVal FileList As List(Of String), ByVal Item As Integer)
        InitializeComponent()

        If FileList Is Nothing Then
            Me.Close()
            Return
        End If
        Me.SetStyle(ControlStyles.Opaque, True)
        PhotoViewerMain1.FileList = FileList
        PhotoViewerMain1.CurItem = Item
        Me.Text = FileList(Item)
    End Sub

    ''' <summary>
    ''' 現在表示しているイメージ
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property PhotoViewerImage() As Image
        Get
            Return PhotoViewerMain1.PhotoViewerImage
        End Get
        Set(ByVal value As Image)
            PhotoViewerMain1.PhotoViewerImage = value
        End Set
    End Property

    ''' <summary>
    ''' ウィンドウが閉じられる時
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub PhotoViewerMain1_CloseEvent() Handles PhotoViewerMain1.CloseEvent
        GC.Collect()
        System.GC.WaitForPendingFinalizers()
        Me.Close()
    End Sub

    ''' <summary>
    ''' イメージに変更がかかった時。更新イベントを発行する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub PhotoViewerMain1_ImageRefreshEvent() Handles PhotoViewerMain1.ImageRefreshEvent
        RaiseEvent ImageRefreshEvent()
    End Sub

    ''' <summary>
    ''' 最大化イベントの処理
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub PhotoViewerMain1_MaximizeEvent() Handles PhotoViewerMain1.MaximizeEvent
        If Me.WindowState = FormWindowState.Maximized Then
            Me.WindowState = FormWindowState.Normal
        Else
            Me.WindowState = FormWindowState.Maximized
        End If
        If Not PhotoViewerMain1.WinState = Me.WindowState Then
            PhotoViewerMain1.WinState = Me.WindowState
        End If
    End Sub

    ''' <summary>
    ''' 最小化イベントの処理
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub PhotoViewerMain1_MinimizeEvent() Handles PhotoViewerMain1.MinimizeEvent
        Me.WindowState = FormWindowState.Minimized
    End Sub

    ''' <summary>
    ''' 角を丸くしたウィンドウの設定
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SetOrgRegion()
        If Me.WindowState = FormWindowState.Minimized Then
            Return
        End If

        Dim iPointRgn As Integer() = {0, 24, 1, 23, 1, 16, 2, 15, 2, 11, 3, 10, 3, 8, 4, 7, 4, 6, 5, 5, 5, 4, 6, 3, 6, 2, 7, 1, 8, 1, 9, 0}
        Dim iPointRgn2 As Integer() = {0, 23, 1, 23, 1, 16, 2, 15, 2, 11, 3, 10, 3, 8, 4, 7, 4, 6, 5, 5, 5, 4, 6, 3, 6, 2, 7, 1, 8, 1, 9, 0}
        Dim i As Integer

        Dim siz As Size = Me.Size
        Dim path As Drawing2D.GraphicsPath = New Drawing2D.GraphicsPath()

        For i = 0 To iPointRgn.Length - 4 Step 2
            path.AddLine(iPointRgn(i), iPointRgn(i + 1), iPointRgn(i + 2), iPointRgn(i + 3))
        Next
        path.AddLine(iPointRgn(iPointRgn.Length - 2), iPointRgn(iPointRgn.Length - 1), siz.Width - iPointRgn(iPointRgn.Length - 2), iPointRgn(iPointRgn.Length - 1))

        For i = iPointRgn.Length - 4 To 0 Step -2
            path.AddLine(siz.Width - iPointRgn(i + 2), iPointRgn(i + 3), siz.Width - iPointRgn(i), iPointRgn(i + 1))
        Next
        path.AddLine(siz.Width - iPointRgn(0), iPointRgn(1), siz.Width - iPointRgn(0), siz.Height - iPointRgn(1) - 1)

        For i = 0 To iPointRgn2.Length - 4 Step 2
            path.AddLine(siz.Width - iPointRgn2(i), siz.Height - iPointRgn2(i + 1) - 1, siz.Width - iPointRgn2(i + 2), siz.Height - iPointRgn2(i + 3) - 1)
        Next
        path.AddLine(siz.Width - iPointRgn2(iPointRgn2.Length - 2), siz.Height - iPointRgn2(iPointRgn2.Length - 1) - 1, iPointRgn2(iPointRgn2.Length - 2), siz.Height - iPointRgn2(iPointRgn2.Length - 1) - 1)

        For i = iPointRgn2.Length - 4 To 0 Step -2
            path.AddLine(iPointRgn2(i + 2), siz.Height - iPointRgn2(i + 3) - 1, iPointRgn2(i), siz.Height - iPointRgn2(i + 1) - 1)
        Next

        If Not DesignMode Then
            Me.Region = New Region(path)
        End If
    End Sub

    ''' <summary>
    ''' サイズ変更時の処理
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PhotoViewerForm_Resize(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        SetOrgRegion()
        PhotoViewerMain1.WinState = Me.WindowState
    End Sub

End Class
