﻿Imports System.ComponentModel

Public Class PhotoViewerMain
#Region "定義・ローカル変数・イベント"
    <Category("PhotoViewerMain")> _
    <Description("閉じるボタンイベント")> _
    Public Event CloseEvent()
    <Category("PhotoViewerMain")> _
    <Description("最小化ボタンイベント")> _
    Public Event MinimizeEvent()
    <Category("PhotoViewerMain")> _
    <Description("最大化ボタンイベント")> _
    Public Event MaximizeEvent()

    <Category("PhotoViewerMain")> _
    <Description("更新イベント")> _
    Public Event ImageRefreshEvent()

    Private prop_Image As Image = Nothing

    Private m_poiPanelBase As Point = New Point(56, 75) ' パネルの標準位置
    Private m_sizSCMargin As Size = New Size(55, 185)

    Private m_bPicMove As Boolean = False ' ピクチャがスクロールできる時に、ドラッグで移動モードになっているか
    Private m_poiMouse As Point

    Const ZOOM_MAX As Double = 16.0 ' 拡大の最大値
    Const IMGSIZE_MAX As Double = 8192.0
    Private m_dZoom As Double = 1.0
    Private m_dStretch As Double = 1.0
    Private m_nStretchMode As Boolean = True ' 伸縮モードか
    Private m_sizPrevImage As Size = Nothing

    Private prop_Comment As String = Nothing ' コメント

    Private prop_FileList As List(Of String) = Nothing ' ファイルパスのリスト
    Private prop_CurItem As Integer = 0 ' 現在の画像index

    Private prop_WinState As FormWindowState = FormWindowState.Normal
#End Region

#Region "システム関数"
    ''' <summary>
    ''' イメージの印刷
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PrintDocument1_PrintPage(ByVal sender As System.Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles PrintDocument1.PrintPage
        If prop_Image Is Nothing Then
            e.Cancel = True
            Return
        End If

        Dim rc As Rectangle = e.MarginBounds
        Dim pr1, pr2 As Double
        pr1 = rc.Height / rc.Width
        pr2 = prop_Image.Height / prop_Image.Width
        ' イメージの縦横比とプリンタの縦横比を比べ、イメージの方が縦に長いのであれば、
        ' 縦に合わせて印刷する
        If pr1 < pr2 Then
            rc.Width = rc.Width * (pr1 / pr2)
            ' 縦長の場合は、中央にあわせてみる
            rc.X = rc.X + (e.MarginBounds.Width - rc.Width) / 2
        Else
            rc.Height = rc.Height * (pr2 / pr1)
        End If

        e.Graphics.DrawImage(prop_Image, rc)
    End Sub

    ''' <summary>
    ''' 印刷プレビューボタン
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BtnPreview_Click() Handles BtnPreview.Click
        PrintPreviewDialog1.Document = PrintDocument1
        PrintPreviewDialog1.ShowDialog()
    End Sub

    ''' <summary>
    ''' 印刷ボタン
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BtnPrint_Click() Handles BtnPrint.Click
        PrintDialog1.Document = PrintDocument1
        If PrintDialog1.ShowDialog() = Windows.Forms.DialogResult.OK Then
            PrintDocument1.PrinterSettings = PrintDialog1.PrinterSettings
            PrintDocument1.Print()
        End If
    End Sub

    ''' <summary>
    ''' 画面読み込み時の初期化処理
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PhotoViewerMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.SetStyle(ControlStyles.UserPaint, True)
        Me.SetStyle(ControlStyles.AllPaintingInWmPaint, True)

        If My.Settings.PictureWindowPos.X >= 0 Then
            If My.Settings.PictureWindowPos.X <= SystemInformation.VirtualScreen.Width Then
                If My.Settings.PictureWindowPos.Y <= SystemInformation.VirtualScreen.Height Then
                    Me.ParentForm.Location = My.Settings.PictureWindowPos
                End If
            End If
        End If

        If My.Settings.PictureWindowSize.Width >= 0 Then
            If My.Settings.PictureWindowSize.Width <= SystemInformation.VirtualScreen.Width Then
                If My.Settings.PictureWindowSize.Height <= SystemInformation.VirtualScreen.Height Then
                    If My.Settings.PictureWindowSize.Width > Me.MinWindowSize.Width Then
                        If My.Settings.PictureWindowSize.Height > Me.MinWindowSize.Height Then
                            Me.ParentForm.Size = My.Settings.PictureWindowSize
                        End If
                    End If
                End If
            End If
        End If

        LoadImage()
        FitToWindow()

        ActiveControl = Panel1
    End Sub

    ''' <summary>
    ''' サイズ変更時の処理
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PhotoViewerMain_Resize(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Resize
        Static s_PrevSize As Size = Nothing
        If Not Me.ClientSize = s_PrevSize Then
            ResizeWindow()
        End If
        s_PrevSize = Me.ClientSize
    End Sub

    ''' <summary>
    ''' 閉じるボタン押下時の処理。設定を保存する
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BtnClose_Click() Handles BtnClose.Click
        My.Settings.PictureWindowSize = Me.ParentForm.Size
        Dim x, y As Integer
        x = Me.ParentForm.Location.X
        y = Me.ParentForm.Location.Y
        If x < 0 Then
            x = 0
        End If
        If y < 0 Then
            y = 0
        End If
        My.Settings.PictureWindowPos = New Point(x, y)
        PhotoViewerImage = Nothing

        RaiseEvent CloseEvent()
    End Sub

    ''' <summary>
    ''' 最大化ボタン押下時の処理
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BtnMaximize_Click() Handles BtnMaximize.Click
        RaiseEvent MaximizeEvent()
    End Sub

    ''' <summary>
    ''' 最小化ボタン押下時の処理
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BtnMinimize_Click() Handles BtnMinimize.Click
        RaiseEvent MinimizeEvent()
    End Sub

    ''' <summary>
    ''' イメージ上でのマウス移動。スクロールバーが出ていればスクロールを行う。無ければウィンドウの移動を行う。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainPicture_MouseMove(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MainPicture.MouseMove
        Dim iXPos As Integer
        Dim iYPos As Integer

        If e.Location = m_poiMouse Then
            Return
        End If

        If m_bPicMove Then
            If Panel1.HorizontalScroll.Visible Or Panel1.VerticalScroll.Visible Then
                iXPos = Panel1.HorizontalScroll.Value - (e.Location.X - m_poiMouse.X)
                If iXPos < 0 Then
                    iXPos = 0
                End If
                If iXPos > Panel1.HorizontalScroll.Maximum Then
                    iXPos = Panel1.HorizontalScroll.Maximum
                End If

                iYPos = Panel1.VerticalScroll.Value - (e.Location.Y - m_poiMouse.Y)
                If iYPos < 0 Then
                    iYPos = 0
                End If
                If iYPos > Panel1.VerticalScroll.Maximum Then
                    iYPos = Panel1.VerticalScroll.Maximum
                End If
                Panel1.AutoScrollPosition = New Point(iXPos, iYPos)
            Else
                Me.Parent.Location += (e.Location - m_poiMouse)
            End If
        End If
    End Sub

    ''' <summary>
    ''' イメージ上でのマウス押下解除時の処理。
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainPicture_MouseUp(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MainPicture.MouseUp
        If (e.Button And Windows.Forms.MouseButtons.Left) = Windows.Forms.MouseButtons.Left Then
            MainPicture.Capture = False
            m_bPicMove = False
            m_poiMouse = e.Location
        End If
    End Sub

    ''' <summary>
    ''' イメージ上でのマウス押下時の処理。スクロール時にはスクロール状態にする
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainPicture_MouseDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MainPicture.MouseDown
        If (e.Button And Windows.Forms.MouseButtons.Left) = Windows.Forms.MouseButtons.Left Then
            MainPicture.Focus()
            m_bPicMove = True
            MainPicture.Capture = True
            m_poiMouse = e.Location
        End If
    End Sub

    ''' <summary>
    ''' イメージからマウスが離れた時の処理。カーソルを標準のものにする
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainPicture_MouseLeave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MainPicture.MouseLeave
        Me.Cursor = Cursors.Arrow
    End Sub

    ''' <summary>
    ''' イメージ上にマウスが来たときの処理。スクロールバーがあったら手カーソルにする
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainPicture_MouseEnter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MainPicture.MouseEnter
        If Panel1.HorizontalScroll.Visible Or Panel1.VerticalScroll.Visible Then
            Me.Cursor = Cursors.Hand
        End If
    End Sub


    ''' <summary>
    ''' コメントのテキスト欄からフォーカスが失われる時、チェックを行う
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub TextBox1_Leave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBox1.Leave
        ActiveControl = Panel1
        CheckComment()

    End Sub

    ''' <summary>
    ''' ダブルクリック。左ボタンなら最大化イベントを発行する
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PhotoViewerMain_MouseDoubleClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDoubleClick
        If e.Button = Windows.Forms.MouseButtons.Left Then
            RaiseEvent MaximizeEvent()
        End If
    End Sub

    ''' <summary>
    ''' 左回転ボタン
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BtnTurnleft_Click() Handles BtnTurnleft.Click
        TurnLeft()
    End Sub

    ''' <summary>
    ''' 右回転ボタン
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BtnTurnright_Click() Handles BtnTurnright.Click
        TurnRight()
    End Sub

    ''' <summary>
    ''' 次の画像ボタン
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BtnNext_Click() Handles BtnNext.Click
        CurItem = CurItem + 1
    End Sub

    ''' <summary>
    ''' 前の画像ボタン
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BtnLast_Click() Handles BtnLast.Click
        CurItem = CurItem - 1
    End Sub

    ''' <summary>
    ''' ウィンドウのサイズに合わせるボタン
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BtnWsize_Click() Handles BtnWsize.Click
        FitToWindow()
    End Sub

    ''' <summary>
    ''' オリジナルのサイズに合わせるボタン
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BtnOrgsize_Click() Handles BtnOrgsize.Click
        FitToImage()
    End Sub

    ''' <summary>
    ''' ズームインボタン
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub Btnminus_Click() Handles Btnminus.Click
        ZoomDown()
    End Sub

    ''' <summary>
    ''' ズームアウトボタン
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub BtnPlus_Click() Handles BtnPlus.Click
        ZoomUp()
    End Sub
#End Region

#Region "ユーザー関数"
    ''' <summary>
    ''' コメントの変更チェック
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub CheckComment()
        If prop_Comment Is Nothing Then
            If TextBox1.Text.Length > 0 Then
                Comment = TextBox1.Text
            End If
        Else
            If Not prop_Comment.CompareTo(TextBox1.Text) = 0 Then
                Comment = TextBox1.Text
            End If
        End If
    End Sub

    ''' <summary>
    ''' 左回転
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub TurnLeft()
        PhotoViewerImage = Nothing
        Try
            System.GC.Collect()
            System.GC.WaitForPendingFinalizers()
            ImageLibs.ImageRotate(FileList(CurItem), RotateFlipType.Rotate270FlipNone)
        Catch ex As Exception
            MessageBox.Show("回転処理中にエラーが発生しました。", My.Application.Info.Title, MessageBoxButtons.OK)
        End Try

        LoadImage()
        FitToWindow()
        MainPicture.Focus()
        RaiseEvent ImageRefreshEvent()
    End Sub

    ''' <summary>
    ''' 右回転
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub TurnRight()
        PhotoViewerImage = Nothing
        Try
            System.GC.Collect()
            System.GC.WaitForPendingFinalizers()
            ImageLibs.ImageRotate(FileList(CurItem), RotateFlipType.Rotate90FlipNone)
        Catch ex As Exception
            MessageBox.Show("回転処理中にエラーが発生しました。", My.Application.Info.Title, MessageBoxButtons.OK)
        End Try
        LoadImage()
        FitToWindow()
        MainPicture.Focus()
        RaiseEvent ImageRefreshEvent()
    End Sub

    ''' <summary>
    ''' 拡大
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ZoomUp()
        Dim dZoomOrg = m_dZoom
        dZoomOrg = dZoomOrg + 0.5
        If dZoomOrg > ZOOM_MAX Then
            dZoomOrg = ZOOM_MAX
        End If

        If Not dZoomOrg = m_dZoom Then
            m_dZoom = dZoomOrg
            Panel1.AutoScroll = True
            ResizeWindow()
        End If
    End Sub

    ''' <summary>
    ''' 縮小
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ZoomDown()
        Dim dZoomOrg = m_dZoom
        dZoomOrg = dZoomOrg - 0.5

        If Not dZoomOrg = m_dZoom Then
            m_dZoom = dZoomOrg

            Dim sizPanel As Size
            sizPanel = New Size(Me.ClientSize.Width - (m_poiPanelBase.X + m_sizSCMargin.Width), Me.ClientSize.Height - (m_poiPanelBase.Y + m_sizSCMargin.Height))

            ' 縮小した結果がウィンドウに収まれば伸縮モードに切り替える
            If MainPicture.Image.Width * m_dZoom * m_dStretch < sizPanel.Width Then
                If MainPicture.Image.Height * m_dZoom * m_dStretch < sizPanel.Height Then
                    m_dZoom = 1.0
                    Dim pr1, pr2 As Double
                    pr1 = sizPanel.Height / sizPanel.Width
                    pr2 = MainPicture.Image.Height / MainPicture.Image.Width
                    If pr1 < pr2 Then
                        m_dStretch = sizPanel.Height / MainPicture.Image.Height
                    Else
                        m_dStretch = sizPanel.Width / MainPicture.Image.Width
                    End If
                    If m_dStretch > 1.0 Then
                        m_dStretch = 1.0
                    End If
                    Panel1.AutoScroll = False
                    m_nStretchMode = True
                End If
            End If

            ResizeWindow()
        End If
    End Sub

    ''' <summary>
    ''' ウィンドウにあわせる
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub FitToWindow()
        If MainPicture.Image Is Nothing Then
            Return
        End If
        m_nStretchMode = True

        Dim sizPanel As Size
        sizPanel = New Size(Me.ClientSize.Width - (m_poiPanelBase.X + m_sizSCMargin.Width), Me.ClientSize.Height - (m_poiPanelBase.Y + m_sizSCMargin.Height))
        m_sizPrevImage = sizPanel
        Panel1.AutoScroll = False

        m_dZoom = 1.0
        Dim pr1, pr2 As Double
        pr1 = sizPanel.Height / sizPanel.Width
        pr2 = MainPicture.Image.Height / MainPicture.Image.Width
        If pr1 < pr2 Then ' イメージが縦に長い場合は縦をベースに縮尺を決める
            m_dStretch = sizPanel.Height / MainPicture.Image.Height
        Else
            m_dStretch = sizPanel.Width / MainPicture.Image.Width
        End If
        If m_dStretch > 1.0 Then
            m_dStretch = 1.0
        End If

        ResizeWindow()
    End Sub

    ''' <summary>
    ''' イメージにあわせる
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub FitToImage()
        If MainPicture.Image Is Nothing Then
            Return
        End If
        Dim sizPanel As Size
        sizPanel = New Size(Me.ClientSize.Width - (m_poiPanelBase.X + m_sizSCMargin.Width), Me.ClientSize.Height - (m_poiPanelBase.Y + m_sizSCMargin.Height))
        m_sizPrevImage = sizPanel
        Panel1.AutoScroll = True

        m_nStretchMode = False
        m_dZoom = 1.0
        m_dStretch = 1.0
        ResizeWindow()
    End Sub

    ''' <summary>
    ''' サイズ調整
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub ResizeWindow()

        Dim poiPanel As Point
        Dim pr1, pr2 As Double
        If MainPicture.Image Is Nothing Then
            Return
        End If

        Panel1.Visible = False

        Dim sizPanel As Size = New Size(Me.ClientSize.Width - (m_poiPanelBase.X + m_sizSCMargin.Width), Me.ClientSize.Height - (m_poiPanelBase.Y + m_sizSCMargin.Height))
        If m_sizPrevImage = Nothing Then
            m_sizPrevImage = sizPanel
        End If

        If m_dStretch * m_dZoom > 1.0 Then
            m_nStretchMode = False
            Panel1.AutoScroll = True
        End If

        If m_nStretchMode Then
            If Not m_sizPrevImage = sizPanel Then
                pr1 = sizPanel.Height / sizPanel.Width
                pr2 = MainPicture.Image.Height / MainPicture.Image.Width
                If pr1 < pr2 Then ' イメージが縦に長い場合は縦をベースに縮尺を決める
                    m_dStretch = sizPanel.Height / MainPicture.Image.Height
                Else
                    m_dStretch = sizPanel.Width / MainPicture.Image.Width
                End If
                If m_dStretch > 1.0 Then
                    m_dStretch = 1.0
                End If
            End If
        End If

        TextBox1.Width = sizPanel.Width
        Label1.MaximumSize = New Size(sizPanel.Width - 80, 21)
        TextBox1.Location = New Point(m_poiPanelBase.X, Me.ClientSize.Height - m_sizSCMargin.Height + 10)

        poiPanel = m_poiPanelBase

        ' 拡大後の画像が一定サイズを超えないようにするための処理
        If MainPicture.Image.Width * m_dStretch * m_dZoom >= IMGSIZE_MAX Then
            m_dZoom = IMGSIZE_MAX / MainPicture.Image.Width / m_dStretch
        End If
        If MainPicture.Image.Height * m_dStretch * m_dZoom >= IMGSIZE_MAX Then
            m_dZoom = IMGSIZE_MAX / MainPicture.Image.Height / m_dStretch
        End If

        MainPicture.Size = New Size(MainPicture.Image.Width * m_dStretch * m_dZoom, MainPicture.Image.Height * m_dStretch * m_dZoom)

        If MainPicture.Width < sizPanel.Width Then
            poiPanel.X = poiPanel.X + (sizPanel.Width - MainPicture.Width) / 2
            sizPanel.Width = MainPicture.Width + System.Windows.Forms.SystemInformation.VerticalScrollBarWidth
        End If
        If MainPicture.Height < sizPanel.Height Then
            poiPanel.Y = poiPanel.Y + (sizPanel.Height - MainPicture.Height) / 2
            sizPanel.Height = MainPicture.Height + System.Windows.Forms.SystemInformation.HorizontalScrollBarHeight
        End If

        Panel1.Location = poiPanel
        Panel1.Size = sizPanel
        Panel1.Visible = True
        m_sizPrevImage = sizPanel
    End Sub

    ''' <summary>
    ''' 画像読み込み
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LoadImage()
        If FileList Is Nothing Then
            Return
        End If
        If CurItem < 0 Then
            Return
        End If
        If FileList.Count < CurItem Then
            Return
        End If
        If IsDBNull(FileList(CurItem)) Then
            Return
        End If

        Try
            PhotoViewerImage = Image.FromFile(FileList(CurItem))
            Label1.Text = System.IO.Path.GetFileName(FileList(CurItem))
        Catch ex As Exception

        End Try
        ResizeWindow()
        TextBox1.Text = Comment
    End Sub
#End Region

#Region "プロパティ"
    ''' <summary>
    ''' 表示するファイルのパスのリスト
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoViewerMain")> _
    <Description("表示するファイルのパスのリスト")> _
    Public Property FileList() As List(Of String)
        Get
            Return prop_FileList
        End Get
        Set(ByVal value As List(Of String))
            prop_FileList = value
            CurItem = -1
        End Set
    End Property

    ''' <summary>
    ''' 現在のイメージ番号
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoViewerMain")> _
    <Description("現在のイメージ番号(FileListに依存)")> _
    <DefaultValue(0)> _
    Public Property CurItem() As Integer
        Get
            Return prop_CurItem
        End Get
        Set(ByVal value As Integer)
            prop_CurItem = value

            If FileList IsNot Nothing Then
                If prop_CurItem > FileList.Count - 1 Then
                    prop_CurItem = 0
                End If
                If prop_CurItem < 0 Then
                    prop_CurItem = FileList.Count - 1
                End If

                Panel1.Visible = False

                CheckComment()
                LoadImage()
                FitToWindow()
                MainPicture.Focus()
            End If
        End Set
    End Property

    ''' <summary>
    ''' コメント
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoViewerMain")> _
    <Description("コメント文字列")> _
    Private Property Comment() As String
        Get
            If FileList Is Nothing Then
                Return Nothing
            End If
            If FileList(CurItem) Is Nothing Then
                Return Nothing
            End If

            Dim strComment As String = Nothing

            Try
                Dim oSummProps As DSOFile.SummaryProperties
                Dim oDocument As DSOFile.OleDocumentPropertiesClass
                oDocument = New DSOFile.OleDocumentPropertiesClass
                oDocument.Open(FileList(CurItem), True, DSOFile.dsoFileOpenOptions.dsoOptionOpenReadOnlyIfNoWriteAccess)
                oSummProps = oDocument.SummaryProperties
                strComment = oSummProps.Comments
                If oSummProps.Comments = Nothing Then

                End If
                oDocument.Close()
            Catch ex As Exception

            End Try

            prop_Comment = strComment
            Return prop_Comment
        End Get
        Set(ByVal value As String)
            If FileList(CurItem) Is Nothing Then
                Return
            End If

            Try
                System.GC.Collect()
                System.GC.WaitForPendingFinalizers()

                Dim oDocument As DSOFile.OleDocumentPropertiesClass
                oDocument = New DSOFile.OleDocumentPropertiesClass

                oDocument.Open(FileList(CurItem), False, DSOFile.dsoFileOpenOptions.dsoOptionDefault)
                Try
                    oDocument.SummaryProperties.Comments = value
                    oDocument.Save()
                    prop_Comment = value
                Catch ex As Exception

                End Try
                oDocument.Close()
                RaiseEvent ImageRefreshEvent()
            Catch ex As Exception

            End Try
        End Set
    End Property

    ''' <summary>
    ''' 表示されているイメージ
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoViewerMain")> _
    <Description("表示されているイメージ")> _
    Public Property PhotoViewerImage() As Image
        Get
            Return prop_Image
        End Get
        Set(ByVal value As Image)
            prop_Image = value
            MainPicture.Image = prop_Image
        End Set
    End Property

    ''' <summary>
    ''' フォームの状態
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoViewerMain")> _
    <Description("フォームの状態")> _
    Public Property WinState() As FormWindowState
        Get
            Return prop_WinState
        End Get
        Set(ByVal value As FormWindowState)
            prop_WinState = value

            If prop_WinState = FormWindowState.Maximized Then
                ' 最大化された時
                BtnMaximize.NormalImage = My.Resources.W_btn_par_n
                BtnMaximize.HoverImage = My.Resources.W_btn_par_f
                BtnMaximize.PressedImage = My.Resources.W_btn_par_p
                Me.SizAble = False
                Me.MovAble = False
            Else
                ' 通常状態の時
                BtnMaximize.NormalImage = My.Resources.W_btn_max_n
                BtnMaximize.HoverImage = My.Resources.W_btn_max_f
                BtnMaximize.PressedImage = My.Resources.W_btn_max_p
                Me.SizAble = True
                Me.MovAble = True
            End If
            Me.Invalidate()
        End Set
    End Property
#End Region


End Class
